/*!
 * Original code by BuilderIO
 * MIT Licensed, Copyright(c) 2021 BuilderIO, see LICENSE.qwik.md for details
 *
 * Credits to the BuilderIO:
 * https://github.com/BuilderIO/qwik/blob/main/packages/qwik/src/cli/utils/app-command.ts
 */
import { existsSync, readFileSync } from "node:fs";
import { dirname, join, resolve } from "node:path";

/**
 * Interface for package.json
 */
export interface PackageJSON {
  name: string;
  version: string;
  dependencies?: { [pkgName: string]: string };
  devDependencies?: { [pkgName: string]: string };
  description?: string;
  scripts?: { [scriptName: string]: string };
  license?: string;
  main: string;
  module?: string;
  types: string;
  type?: string;
  files?: string[];
  exports?: { [key: string]: any };
  contributors?: { [key: string]: string }[];
  homepage?: string;
  repository?: { [key: string]: string };
  bugs?: { [key: string]: string };
  keywords?: string[];
  engines?: { [key: string]: string };
  private?: boolean;
  [key: string]: any;
}

export class AppCommand {
  args: string[];
  task: string;
  cwd: string;
  private _rootDir: string | undefined;
  private _rootPkgJson: PackageJSON | undefined;

  constructor(opts: { rootDir: string; cwd: string; args: string[] }) {
    this._rootDir = opts.rootDir;
    this.cwd = opts.cwd;
    this.args = opts.args.slice();
    this.task = this.args[0];
  }

  get rootDir() {
    if (!this._rootDir) {
      const fsRoot = resolve("/");
      let testDir = process.cwd();
      for (let i = 0; i < 20; i++) {
        const pkgPath = join(testDir, "package.json");
        if (existsSync(pkgPath)) {
          this._rootDir = testDir;
          break;
        }
        if (testDir === fsRoot) {
          break;
        }
        testDir = dirname(testDir);
      }
      if (!this._rootDir) {
        throw new Error(`Unable to find Qwik app package.json`);
      }
    }
    return this._rootDir;
  }

  set rootDir(rootDir) {
    this._rootDir = rootDir;
  }

  get packageJson(): PackageJSON {
    if (!this._rootPkgJson) {
      const pkgJsonPath = join(this.rootDir, "package.json");
      this._rootPkgJson = JSON.parse(readFileSync(pkgJsonPath, "utf-8"));
    }
    return this._rootPkgJson!;
  }
}
